#include "windows.h"
#include "commctrl.h"
#include "tapi.h"
#include "stdlib.h"
#include "cpl.h"
#include "resource.h"

// ============================================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
void CALLBACK MdmCallback( DWORD, DWORD, DWORD, DWORD, DWORD, DWORD );
int ListDevices( HWND, HLINEAPP * );
BOOL GetDeviceInfo( HWND );
int mbtowc( char *, TCHAR * );
BOOL AppendEditText( HWND, TCHAR * );
BOOL SendRecvSerial( HWND, HANDLE, char *, int, BOOL );
BOOL CenterWindow( HWND );

// ============================================================================

#define SETLINE(h,x)  SetWindowLong(h,DWL_USER,(DWORD)x)
#define GETLINE(h)    (HLINEAPP)GetWindowLong(h,DWL_USER)

typedef struct {
	VARSTRING vs;
	HANDLE hPort;
	char *pszStr;
} VSTRUCT, *PVSTRUCT;

// ============================================================================

HINSTANCE hInst;
DWORD dwAPI;

// ============================================================================

BOOL WINAPI DllMain( HANDLE hModule, ULONG ulReason, PCONTEXT pctx )
{
	if( ulReason == DLL_PROCESS_ATTACH )
		hInst = hModule;
    return( TRUE );
}

// ============================================================================

LONG CALLBACK CPlApplet( HWND hWndCPL, UINT uMsg, LONG lParam1, LONG lParam2 )
{
	LPNEWCPLINFO lpNewCPlInfo;

    switch( uMsg )
	{
		case CPL_INIT:
			return( TRUE );


        case CPL_GETCOUNT:
            return( 1 );


        case CPL_NEWINQUIRE:
		{
			lpNewCPlInfo = (LPNEWCPLINFO)lParam2;

            lpNewCPlInfo->dwSize = (DWORD)sizeof(NEWCPLINFO);
            lpNewCPlInfo->dwFlags = 0;
            lpNewCPlInfo->dwHelpContext = 0;
            lpNewCPlInfo->hIcon = LoadIcon( hInst, 
											MAKEINTRESOURCE( IDI_MDMINFO ) );
			lpNewCPlInfo->lData = IDI_MDMINFO;
            lpNewCPlInfo->szHelpFile[ 0 ] = '\0';
			LoadString( hInst, IDS_NAME, lpNewCPlInfo->szName, 32 );
			LoadString( hInst, IDS_DESC, lpNewCPlInfo->szInfo, 64 );
            break;
		}

        
		case CPL_DBLCLK:
			DialogBox( hInst, MAKEINTRESOURCE( IDD_MAINDLG ), hWndCPL,
					   MainDlgProc );
            break;


		case CPL_SELECT:
		case CPL_STOP:
		case CPL_EXIT:
			break;

		default:
			break;
    }
    return( 0 );
}

// ============================================================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
		{
			HLINEAPP hLine;

			SETLINE( hDlg, 0 );
			CenterWindow( hDlg );
			if( ListDevices( GetDlgItem( hDlg, IDC_DEVICE ), &hLine ) )
				SendDlgItemMessage( hDlg, IDC_DEVICE, CB_SETCURSEL, 0, 0 );
			if( hLine )
				SETLINE( hDlg, hLine );
			return( TRUE );
		}


		case WM_COMMAND:
			if( wParam == IDCANCEL )
			{
				if( GETLINE( hDlg ) )
					lineShutdown( GETLINE( hDlg ) );
				DestroyWindow( hDlg );
				PostQuitMessage( 0 );
				return( TRUE );
			}
			if( wParam == IDC_FIND )
			{
				HLINEAPP hLine;
				int iDevices;


				if( GETLINE( hDlg ) )
					lineShutdown( GETLINE( hDlg ) );
				SETLINE( hDlg, 0 );

				iDevices = ListDevices( GetDlgItem( hDlg, IDC_DEVICE ),
										&hLine );
				if( hLine )
					SETLINE( hDlg, hLine );

				if( hLine == NULL )
					MessageBox( hDlg, L"Device query failed!", L"Modem Test",
								MB_OK | MB_ICONEXCLAMATION );
				else
					SendDlgItemMessage( hDlg, IDC_DEVICE, CB_SETCURSEL,
										0, 0 );
				return( TRUE );
			}
			if( wParam == IDOK )
			{
				if( GETLINE( hDlg ) )
					GetDeviceInfo( hDlg );
				else
					MessageBox( hDlg, L"Select a device first", L"Modem Test",
								MB_OK | MB_ICONEXCLAMATION );
				break;
			}
			break;
	}
	return( FALSE );
}

// ============================================================================

int ListDevices( HWND hCombo, HLINEAPP *hLineApp )
{
	LONG lStatus;
	int iDevices, i, iItem;
	HLINEAPP hLine;
	LINEEXTENSIONID le;
	LPLINEDEVCAPS lpLdc;


	*hLineApp = NULL;
	SendMessage( hCombo, CB_RESETCONTENT, 0, 0 );

	lpLdc = (LPLINEDEVCAPS)malloc( 1024 );
	if( ! lpLdc )
		return( 0 );

	lStatus = lineInitialize( &hLine, hInst, MdmCallback, NULL, &iDevices );
	if( lStatus == 0 )
	{
		*hLineApp = hLine;

		for( i = 0; i < iDevices; i++ )
		{
			lStatus = lineNegotiateAPIVersion( hLine, i, 0x00010000, 
											   0x00010005, &dwAPI, &le );
			if( lStatus == 0 )
			{
				lpLdc->dwTotalSize = 1024;
				lStatus = lineGetDevCaps( hLine, i, dwAPI, 0, lpLdc );
				if( lStatus == 0 )
				{
					iItem = SendMessage( hCombo, CB_ADDSTRING, 0,
					     (LPARAM)( (LPBYTE)lpLdc + lpLdc->dwLineNameOffset ) );
					SendMessage( hCombo, CB_SETITEMDATA, iItem, i );
				}
			}
		}
	}
	free( lpLdc );
	return( iDevices );
}

// ============================================================================

void CALLBACK MdmCallback( DWORD dwDevice, DWORD msg, DWORD dwInst,
						   DWORD dwParam1, DWORD dwParam2, DWORD dwParam3 )
{
	return;
}

// ============================================================================

BOOL GetDeviceInfo( HWND hDlg )
{
	int iItem, iDevice;
	LONG lStatus;
	LPLINEDEVCAPS lpLdc;
	HLINE hLine;
	HCALL hCall;
	LINECALLPARAMS lcp;
	TCHAR szStr[ 128 ];
	HANDLE hPort;
	PVSTRUCT pVs;
	HWND hEdit;


	iItem = SendDlgItemMessage( hDlg, IDC_DEVICE, CB_GETCURSEL, 0, 0 );
	if( iItem == CB_ERR )
		return( FALSE );

	SendDlgItemMessage( hDlg, IDC_INFO, WM_SETTEXT, 0, (LPARAM)L"" );
	hEdit = GetDlgItem( hDlg, IDC_INFO );

	lpLdc = (LPLINEDEVCAPS)malloc( 1024 );
	if( ! lpLdc )
		return( FALSE );

	lpLdc->dwTotalSize = 1024;
	iDevice = SendDlgItemMessage( hDlg, IDC_DEVICE, CB_GETITEMDATA, iItem, 0 );
	lStatus = lineGetDevCaps( GETLINE( hDlg ), iDevice, dwAPI, 0, lpLdc );
	if( lStatus == 0 )
	{
		lStatus = lineOpen( GETLINE( hDlg ), iDevice, &hLine, dwAPI, 0, 2,
							LINECALLPRIVILEGE_OWNER,
							LINEMEDIAMODE_DATAMODEM, NULL );
		if( lStatus == 0 )
		{
			AppendEditText( hEdit, L"<<< Line to device opened >>>\r\n" );

			memset( &lcp, 0, sizeof(LINECALLPARAMS) );
			lcp.dwTotalSize = sizeof(LINECALLPARAMS);
			lcp.dwMaxRate = 1200;
			lcp.dwBearerMode = LINEBEARERMODE_PASSTHROUGH;
			lcp.dwMediaMode = LINEMEDIAMODE_UNKNOWN;
			lStatus = lineMakeCall( hLine, &hCall, NULL, 0, &lcp );
			if( lStatus >= 0 )
			{
				AppendEditText( hEdit, L"<<< Device opened >>>\r\n" );

				pVs = (PVSTRUCT)szStr;
				pVs->vs.dwTotalSize = 128;
				lStatus = lineGetID( hLine, iDevice, hCall,
									 LINECALLSELECT_CALL,
									 &pVs->vs, L"comm/datamodem" );
				if( lStatus == 0 )
				{
					hPort = pVs->hPort;
					if( hPort )
					{
						AppendEditText( hEdit, L"<<< Talking to device >>>\r\n" );
						
						AppendEditText( hEdit, L"\r\nATI0: " );
						SendRecvSerial( hEdit, hPort, "ATI0\r\n", 6, TRUE );
						
						AppendEditText( hEdit, L"ATI1: " );
						SendRecvSerial( hEdit, hPort, "ATI1\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI2: " );
						SendRecvSerial( hEdit, hPort, "ATI2\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI3: " );
						SendRecvSerial( hEdit, hPort, "ATI3\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI4: " );
						SendRecvSerial( hEdit, hPort, "ATI4\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI5: " );
						SendRecvSerial( hEdit, hPort, "ATI5\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI6: " );
						SendRecvSerial( hEdit, hPort, "ATI6\r\n", 6, TRUE );

						AppendEditText( hEdit, L"ATI7: " );
						SendRecvSerial( hEdit, hPort, "ATI7\r\n", 6, TRUE );

						AppendEditText( hEdit, L"AT+FCLASS=?: " );
						SendRecvSerial( hEdit, hPort, "AT+FCLASS=?\r\n", 13, TRUE );

						CloseHandle( hPort );

						AppendEditText( hEdit, L"\r\n<<< Device closed >>>\r\n" );
					}
				}
				else
				{
					wsprintf( szStr, L"\r\n<<< lineGetID() error %.8x >>>\r\n", lStatus );
					AppendEditText( hEdit, szStr );
				}

				lineDrop( hCall, NULL, 0 );
			}
			else
			{
				wsprintf( szStr, L"<<< \r\nlineMakeCall() error %.8x >>>\r\n", lStatus );
				AppendEditText( hEdit, szStr );
			}
		
			lineClose( hLine );
		}
		else
		{
			wsprintf( szStr, L"\r\n<<< lineOpen() error %.8x >>>\r\n", lStatus );
			AppendEditText( hEdit, szStr );
		}
	}
	else
		AppendEditText( hEdit, L"\r\n<<< lineGetDevCaps() failed >>>\r\n" );

	free( lpLdc );
	return( TRUE );
}

// ============================================================================

int mbtowc( char *szMByte, TCHAR *szWide )
{
	while( *szMByte )
		*szWide++ = (USHORT)( (BYTE)*szMByte++ & 0xFF );
	*szWide = L'\0';
	return( wcslen( szWide ) );
}

// ============================================================================

BOOL AppendEditText( HWND hEdit, TCHAR *pszStr )
{
	int iLen;


	iLen = SendMessage( hEdit, WM_GETTEXTLENGTH, 0, 0 );
	SendMessage( hEdit, EM_SETSEL, iLen, iLen );
	SendMessage( hEdit, EM_REPLACESEL, 0, (LPARAM)pszStr );
	return( TRUE );
}

// ============================================================================

BOOL SendRecvSerial( HWND hEdit, HANDLE hPort, char *pszSend, int iLen,
					 BOOL bShow )
{
	BOOL bStatus;
	TCHAR szBuffer[ 256 * sizeof(TCHAR) ];
	char pcBuffer[ 256 ], *pscStart, *pscEnd;
	int iOffset;
	DWORD dwOut;


	bStatus = FALSE;
	if( WriteFile( hPort, pszSend, iLen, &dwOut, NULL ) )
	{
		Sleep( 1000 );
		memset( pcBuffer, 0, 256 );
		iOffset = 0;

		while( 1 )
		{
			ReadFile( hPort, pcBuffer + iOffset++, 1, &dwOut, NULL );
			if( ! dwOut )
				break;
		}

		if( bShow && iOffset )
		{
			pscStart = strchr( pcBuffer, '\n' );
			if( ! pscStart )
				pscStart = pcBuffer;
			else
				pscStart++;

			pscEnd = strstr( pcBuffer, "\r\nOK" );
			if( pscEnd )
				*pscEnd = '\0';

			mbtowc( pscStart, szBuffer );
			AppendEditText( hEdit, szBuffer );
		}
	}
	return( bStatus );
}

// ============================================================================

BOOL CenterWindow( HWND hWnd )
{
	int iXsize, iYsize;
	RECT Rect;

	iXsize = GetSystemMetrics( SM_CXSCREEN );
	iYsize = GetSystemMetrics( SM_CYSCREEN );
	GetWindowRect( hWnd, &Rect );
	Rect.right = Rect.right - Rect.left;
	Rect.bottom = Rect.bottom - Rect.top;
	Rect.left = ( iXsize - Rect.right ) / 2;
	Rect.top = ( iYsize - Rect.bottom ) / 2;
	SetWindowPos( hWnd, NULL, Rect.left, 0, 0, 0,
				  SWP_NOSIZE | SWP_NOZORDER );
	return( TRUE );
}

// ============================================================================


